const Me = imports.misc.extensionUtils.getCurrentExtension();
const { Gio, GLib, GObject, St } = imports.gi;
const { popupMenu } = imports.ui;
const { QuickMenuToggle, SystemIndicator } = imports.ui.quickSettings;
const Log = Me.imports.modules.log;
const Resources = Me.imports.modules.resources;
var Title = "supergfxctl";
const gfxLabels = [
    "hybrid",
    "dedicated",
    "integrated",
    "compute",
    "vfio",
    "egpu",
    "unknown"
];
const gfxLabelsMenu = [
    "Hybrid",
    "Dedicated",
    "Integrated",
    "Compute",
    "VFIO",
    "e-GPU",
    "unknown"
];
const powerLabel = ["active", "suspended", "off", "unknown"];
const userAction = ["logout", "reboot", "integrated", "none"];
const acls = [
    [true, true, true, false, false, true],
    [true, true, true, false, false, true],
    [true, true, true, true, true, true],
    [true, false, true, true, true, false],
    [true, false, true, true, true, false],
    [true, true, true, true, true, true]
];
var gpuModeIndicator43 = GObject.registerClass(class gpuModeIndicator43 extends SystemIndicator {
    _init() {
        super._init();
        this._indicator = this._addIndicator();
        this._indicator.visible = true;
        this.quickSettingsItems.push(quickToggles);
    }
});
var gpuModeIndicator = GObject.registerClass(class gpuModeIndicator extends QuickMenuToggle {
    constructor() {
        super(...arguments);
        this.connected = false;
        this.pollerDelayTicks = 0;
        this.timeoutPollerGpuPower = null;
        this._supported = [];
        this._vendor = "";
        this._lastState = 6;
        this._powerState = 3;
    }
    _init() {
        super._init();
        this._gpuModeItems = new Map();
        this._updateMenu = true;
        this._createProxy();
    }
    _createProxy() {
        let xml = Resources.File.DBus("org-supergfxctl-gfx-4");
        this._proxy = new Gio.DBusProxy.makeProxyWrapper(xml)(Gio.DBus.system, "org.supergfxctl.Daemon", "/org/supergfxctl/Gfx", (proxy, e) => {
            if (e) {
                Log.error("Graphics Mode DBus initialization using supergfxctl failed!", e);
            }
            else {
                try {
                    this.connected = true;
                    this.connectSignal();
                    this.getSupported();
                    this.getVendor();
                    this.getGfxMode();
                    this.getGpuPower();
                    this._gpuModeSection = new popupMenu.PopupMenuSection();
                    this.menu.addMenuItem(this._gpuModeSection);
                    this.label = `${gfxLabelsMenu[this._lastState]} (${powerLabel[this._powerState]})`;
                    this.gicon = Gio.icon_new_for_string(`${Me.path}/icons/scalable/gpu-${gfxLabels[this._lastState]}${(powerLabel[this._powerState] == 'active' ? '-active' : '')}.svg`);
                    this.menu.setHeader(Gio.icon_new_for_string(`${Me.path}/icons/scalable/gpu-${gfxLabels[this._lastState]}${(powerLabel[this._powerState] == 'active' ? '-active' : '')}.svg`), `${gfxLabelsMenu[this._lastState]} (${powerLabel[this._powerState]})`);
                    this._sync();
                }
                catch (e) {
                    Log.error("Graphics Mode DBus initialization using supergfxctl failed after making the connection!", e);
                }
            }
        });
    }
    _sync() {
        if (this._updateMenu) {
            this._gpuModeSection.removeAll();
            this._gpuModeItems.clear();
            for (const key in this._supported) {
                if (Object.prototype.hasOwnProperty.call(this._supported, key)) {
                    const element = this._supported[key];
                    if (gfxLabels[element] == 'dedicated' &&
                        this._vendor == 'AMD')
                        continue;
                    const item = new popupMenu.PopupImageMenuItem(gfxLabelsMenu[element], Gio.icon_new_for_string(`${Me.path}/icons/scalable/gpu-${gfxLabels[element]}.svg`));
                    item.connect("activate", () => {
                        this.setGfxMode(element);
                    });
                    this._gpuModeItems.set(element, item);
                    this._gpuModeSection.addMenuItem(item);
                }
            }
            this._gpuModeSection.addMenuItem(new popupMenu.PopupSeparatorMenuItem());
            this._gpuModeSection.addMenuItem(new popupMenu.PopupImageMenuItem(`dedicated GPU is ${powerLabel[this._powerState]}`, Gio.icon_new_for_string(`${Me.path}/icons/scalable/dgpu-${powerLabel[this._powerState]}.svg`)));
            this.label = `${gfxLabelsMenu[this._lastState]} (${powerLabel[this._powerState]})`;
            this.gicon = Gio.icon_new_for_string(`${Me.path}/icons/scalable/gpu-${gfxLabels[this._lastState]}${(powerLabel[this._powerState] == 'active' ? '-active' : '')}.svg`);
            this.setIndicator();
            this._updateMenu = false;
        }
        for (const [profile, item] of this._gpuModeItems) {
            item.setOrnament(profile === this._lastState
                ? popupMenu.Ornament.DOT
                : popupMenu.Ornament.NONE);
        }
        this.menu.setHeader(Gio.icon_new_for_string(`${Me.path}/icons/scalable/gpu-${gfxLabels[this._lastState]}${(powerLabel[this._powerState] == 'active' ? '-active' : '')}.svg`), `${gfxLabelsMenu[this._lastState]} (${powerLabel[this._powerState]})`);
    }
    setIndicator() {
        ext.gpuModeIndicatorInstance.remove_actor(ext.gpuModeIndicatorInstance._indicator);
        ext.gpuModeIndicatorInstance._indicator = new St.Icon({
            gicon: Gio.icon_new_for_string(`${Me.path}/icons/scalable/gpu-${gfxLabels[quickToggles._lastState]}${(powerLabel[quickToggles._powerState] == 'active' ? '-active' : '')}.svg`),
            style_class: 'supergfxctl-gex panel-icon system-status-icon'
        });
        ext.gpuModeIndicatorInstance.add_actor(ext.gpuModeIndicatorInstance._indicator);
    }
    connectSignal() {
        this._proxy.connectSignal("NotifyAction", (proxy = null, name, value) => {
            let newMode = parseInt(this._proxy.ModeSync());
            let msg = `Graphics Mode has changed.`;
            if (userAction[value] === 'integrated') {
                msg = `You must switch to Integrated mode before switching to Compute or VFIO.`;
                newMode = this.lastState;
            }
            else if (userAction[value] !== 'none') {
                msg = `Graphics Mode changed. Please save your work and ${userAction[value]} to apply the changes.`;
                if (newMode !== this.lastState)
                    this.lastState = newMode;
            }
            else {
                if (newMode !== this.lastState) {
                    msg = `Graphics Mode changed to ${this.gfxLabels[newMode]}`;
                    this.lastState = newMode;
                }
            }
            this._sync();
            ext.notify(ext.title, msg, `scalable/notification-${userAction[value]}.svg`, userAction[value]);
        });
        try {
            this.timeoutPollerGpuPower = GLib.timeout_add_seconds(GLib.PRIORITY_DEFAULT, 1, this.pollerGpuPower.bind(this));
        }
        catch (e) {
            Log.error(`Graphics Mode DBus power mode Poller initialization failed!`, e);
        }
    }
    getSupported() {
        if (!this.connected)
            this._createProxy();
        if (this.connected) {
            try {
                let supported = this._proxy.SupportedSync();
                for (const [_key, _value] of Object.entries(supported)) {
                    if (typeof _value == "object") {
                        for (const [_keyInner, _valueInner] of Object.entries(_value)) {
                            this._supported[parseInt(_keyInner)] = parseInt(_valueInner);
                        }
                    }
                }
                return true;
            }
            catch (e) {
                Log.error("Graphics Mode DBus: get current mode failed!", e);
            }
        }
        return false;
    }
    getVendor() {
        if (!this.connected)
            this._createProxy();
        if (this.connected) {
            try {
                this._vendor = this._proxy.VendorSync();
                return this._vendor;
            }
            catch (e) {
                Log.error("Graphics Mode DBus: get current vendor failed!", e);
            }
        }
        return "";
    }
    getGfxMode() {
        if (!this.connected)
            this._createProxy();
        if (this.connected) {
            try {
                this._lastState = parseInt(this._proxy.ModeSync());
                return this._lastState;
            }
            catch (e) {
                Log.error("Graphics Mode DBus: get current mode failed!", e);
            }
        }
        return 6;
    }
    setGfxMode(mode) {
        if (!this.connected)
            this._createProxy();
        if (this.connected) {
            try {
                this._proxy.SetModeSync(mode);
                return true;
            }
            catch (e) {
                Log.error("Graphics Mode DBus switching failed!", e);
                ext.notify(ext.title, e.toString(), 'gif/fire.gif');
            }
        }
        return false;
    }
    getGpuPower() {
        if (!this.connected)
            this._createProxy();
        if (this.connected) {
            try {
                this._powerState = parseInt(this._proxy.PowerSync().toString().trim());
                return true;
            }
            catch (e) {
                Log.error("Graphics Mode DBus getting power mode failed!", e);
            }
        }
        this._powerState = 3;
        return false;
    }
    pollerGpuPower() {
        if (!this.connected)
            this._createProxy();
        if (this.connected && this.pollerDelayTicks <= 0) {
            try {
                this.getGpuPower();
                this.updatePanelPower();
            }
            catch (e) {
                Log.error(`Graphics Mode DBus power mode poller init failed!`, e);
            }
            finally {
                return this.connected ? GLib.SOURCE_CONTINUE : GLib.SOURCE_REMOVE;
            }
        }
        else if (this.connected && this.pollerDelayTicks > 0) {
            this.pollerDelayTicks--;
            return GLib.SOURCE_CONTINUE;
        }
        else {
            return GLib.SOURCE_REMOVE;
        }
    }
    updatePanelPower(gpuPowerLocal = this._powerState) {
        if (gpuPowerLocal !== this.lastStatePower) {
            this.lastStatePower = gpuPowerLocal;
            if (gpuPowerLocal == 0 && this.lastState == 1) {
                try {
                    let mode = parseInt(this._proxy.ModeSync());
                    if (this.gfxLabels[mode] == "integrated")
                        ext.notify(ext.title, `Your dedicated GPU turned on while you are on the integrated mode. This should not happen. It could be that another application rescanned your PCI bus. Rebooting is advised.`, "gif/fire.gif", "reboot");
                    else if (this.lastState !== mode)
                        this.lastState = mode;
                }
                catch (e) {
                    Log.error("Graphics Mode DBus getting mode failed!", e);
                }
            }
            this._updateMenu = true;
            this._sync();
        }
    }
    stop() {
        if (this.connected) {
            this.connected = false;
            GLib.Source.remove(this.timeoutPollerGpuPower);
            this.timeoutPollerGpuPower = null;
            this._proxy = null;
        }
    }
});
var quickToggles = new gpuModeIndicator();
//# sourceMappingURL=panel43.js.map